-- =====================================================================
-- Copyright © 2010-2012 by Cryptographic Engineering Research Group (CERG),
-- ECE Department, George Mason University
-- Fairfax, VA, U.S.A.
-- =====================================================================

library ieee;
use ieee.std_logic_1164.ALL; 
use work.sha3_pkg.all;

entity wrapper is
	generic ( 			
		fifodepth		: integer := 128;	
		uf			: integer := 8;	   
		ff			: integer := 1;
		ppl 		: integer := 10;
		hashsize 	: integer := 256;
		w 			: integer := 64		
	);
	port (					  		
		rst 		: in std_logic;
		clk 		: in std_logic;			
		-- input fifo signals
		ext_idata   : in std_logic_vector(w-1 downto 0);
		fifoin_write : in std_logic_vector(ppl-1 downto 0);
		fifoin_full : out std_logic_vector(ppl-1 downto 0);		
		-- output fifo signals
		fifoout_read : in std_logic;		
		ext_odata   	: out std_logic_vector(w-1 downto 0);
		fifoout_empty 	: out std_logic;	   
		-- simulation 
		hold_data : in std_logic
	);   
end entity wrapper;

architecture structure of wrapper is
	-- SIGNALS BETWEEN HASH CORE AND INPUT FIFO
	signal din : std_logic_vector(ppl*w-1 downto 0);
	signal src_ready, src_read : std_logic_vector(ppl-1 downto 0);
	signal src_ready_simulated : std_logic_vector(ppl-1 downto 0);
	-- SIGNALS BETWEEN HASH CORE AND OUTPUT FIFO
	signal dout : std_logic_vector(w-1 downto 0);
	signal dst_ready, dst_write : std_logic;
begin			
	
	-- FIFO WRAPPERS
	fin: for i in 0 to ppl-1 generate
		fifoin: entity work.fifo_ppl(struct)   
			generic map ( depth => fifodepth, N => W)
			port map (	clk=>clk,		rst=>rst,			write=>fifoin_write(i),	read=>src_read(i),	
						din=>ext_idata,	dout=>din(ppl*w-i*w-1 downto ppl*w-(i+1)*w),		full=>fifoin_full(i),	empty=>src_ready(i));			 
	end generate;
	
	fifoout: entity work.fifo_ppl(struct) 
		generic map ( depth => fifodepth, N => W)
		port map (
		clk=>clk,
		rst=>rst,
		write=>dst_write,
		read=>fifoout_read,
		din=>dout,
		dout=>ext_odata,
		full=>dst_ready,
		empty=>fifoout_empty);
	
	-- SIMULATION
	src_ready_simulated <= (others => '1') when hold_data = '1' else src_ready;	

	-- HASH CORE
	sha3core:  entity work.skein_top_ppl(structure)
		generic map ( UF => UF, PPL => PPL, HS => hashsize )
		port map (		
			-- global
			rst 	=> rst,
			clk 	=> clk,
			
			--fifo
			din			=> din,
			src_read	=> src_read,
			src_ready	=> src_ready_simulated,
			dout		=> dout,
			dst_write	=> dst_write,
			dst_ready	=> dst_ready
	);   
	
end structure;